#property link          "https://www.earnforex.com/metatrader-expert-advisors/supertrend-trailing-stop/"
#property version       "1.03"
#property strict
#property copyright     "EarnForex.com - 2019-2021"
#property description   "This Expert Advisor will Trail the Stop Loss Price" 
#property description   "following the Supertrend Line."
#property description   " "
#property description   "WARNING : You use this software at your own risk."
#property description   "The creator of these plugins cannot be held responsible for any damage or loss."
#property description   " "
#property description   "Find More on EarnForex.com"
#property icon          "\\Files\\EF-Icon-64x64px.ico"

#include <MQLTA ErrorHandling.mqh>
#include <MQLTA Utils.mqh>


enum ENUM_CONSIDER{
   All=-1,        //ALL ORDERS
   Buy=OP_BUY,    //BUY ONLY
   Sell=OP_SELL,  //SELL ONLY
};

enum ENUM_CUSTOMTIMEFRAMES{
   CURRENT=PERIOD_CURRENT,             //CURRENT PERIOD
   M1=PERIOD_M1,                       //M1
   M5=PERIOD_M5,                       //M5
   M15=PERIOD_M15,                     //M15
   M30=PERIOD_M30,                     //M30
   H1=PERIOD_H1,                       //H1
   H4=PERIOD_H4,                       //H4
   D1=PERIOD_D1,                       //D1
   W1=PERIOD_W1,                       //W1
   MN1=PERIOD_MN1,                     //MN1
};


input string Comment_1="====================";     //Expert Advisor Settings
extern double ATRMultiplier=2.0;                   //ATR Multiplier
extern int ATRPeriod=100;                          //ATR Period
input int ATRMaxBars=100;                         //ATR Max Bars (Max 10.000)
input ENUM_CUSTOMTIMEFRAMES StopATRTimeframe=CURRENT;   //ATR Supertrend Timeframe Calculation
input string Comment_2="====================";           //Orders Filtering Options
input bool OnlyCurrentSymbol=true;                       //Apply To Current Symbol Only
input ENUM_CONSIDER OnlyType=All;                        //Apply To
input bool UseMagic=false;                               //Filter By Magic Number
input int MagicNumber=0;                                 //Magic Number (if above is true)
input bool UseComment=false;                             //Filter By Comment
input string CommentFilter="";                           //Comment (if above is true)
extern bool EnableTrailing=false;                        //Enable Trailing Stop
input string Comment_3="====================";     //Notification Options
extern bool EnableNotify=false;                    //Enable Notifications feature
extern bool SendAlert=true;                        //Send Alert Notification
extern bool SendApp=true;                          //Send Notification to Mobile
extern bool SendEmail=true;                        //Send Notification via Email
input string Comment_3a="====================";           //Graphical Window
extern bool ShowPanel=true;                              //Show Graphical Panel
extern string IndicatorName="MQLTA-STTS";          //Indicator Name (to name the objects)
extern int Xoff=20;                                //Horizontal spacing for the control panel
extern int Yoff=20;                                //Vertical spacing for the control panel

int OrderOpRetry=5;
int SuperTrendShift=0;

double TrendUpTmp[], TrendDownTmp[];
int changeOfTrend;
int MaxBars=ATRMaxBars;


int OnInit(){
   CleanPanel();

   if(ShowPanel) DrawPanel();

   return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason){
   CleanPanel();
   
}

void OnTick(){
   CleanPanel();

   if(EnableTrailing) TrailingStop();
   if(ShowPanel) DrawPanel();
   
}


void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam){

   if(id==CHARTEVENT_OBJECT_CLICK){
      if(sparam==PanelEnableDisable){
         ChangeTrailingEnabled();
      }
   }
   if(id==CHARTEVENT_KEYDOWN){
      if(lparam==27){
         if(MessageBox("Are you sure you want to close the EA?","EXIT ?",MB_YESNO)==IDYES){
            ExpertRemove();
         }
      }
   }
   
}

//Print information about author and code


double GetATRTrend(string Instrument="", int Timeframe=0, int Shift=0){
   ArrayResize(TrendDownTmp,ATRMaxBars,0);
   ArrayInitialize(TrendDownTmp,0);
   ArrayResize(TrendUpTmp,ATRMaxBars,0);
   ArrayInitialize(TrendUpTmp,0);
   if(Instrument=="") Instrument=Symbol();
   if(Timeframe==0) Timeframe=Period();
   CalculateSupertrendTmp(Instrument, Timeframe);
   double ATRTrend1=TrendUpTmp[Shift];
   double ATRTrend2=TrendDownTmp[Shift];
   //Print(Timeframe," ",ATRTrend1," ",ATRTrend2);
   if(ATRTrend1>(iClose(Instrument,Timeframe,Shift)*2)){
      return NormalizeDouble(ATRTrend2, (int)MarketInfo(Instrument,MODE_DIGITS));
   }
   if(ATRTrend2>(iClose(Instrument,Timeframe,Shift)*2)){
      return NormalizeDouble(ATRTrend1, (int)MarketInfo(Instrument,MODE_DIGITS));
   }
   if(ATRTrend1==0){
      Print("Please Check That you Have Installed the Indicator MQLTA Supertrend Line");
   }
   return 0;
}


void CalculateSupertrendTmp(string Instrument, int Timeframe){
   MaxBars=ATRMaxBars;
   int limit, i, flag, flagh, trend[10000];
   double up[10000], dn[10000], medianPrice, atr;
   int counted_bars = 0;
   if(counted_bars < 0) return;
   if(counted_bars > 0) counted_bars--;
   limit=iBars(Instrument,Timeframe)-counted_bars-1;
   MaxBars--;
   if(iBars(Instrument,Timeframe)<MaxBars+2+ATRPeriod) MaxBars=iBars(Instrument,Timeframe)-2-ATRPeriod;
   if(MaxBars<=0){
      Print("Need more historical data to calculate the Supertrend, Currently have only ",iBars(Instrument,Timeframe)," Bars");
      return;
   }
   for(i=MaxBars;i>=0;i--) {
      //Print(Bars," ",i);
      TrendUpTmp[i]=EMPTY_VALUE;
      TrendDownTmp[i]=EMPTY_VALUE;
      atr=iATR(Instrument,Timeframe,ATRPeriod,i);
      //Print("atr: "+atr[i]);
      medianPrice = (iHigh(Instrument,Timeframe,i)+iLow(Instrument,Timeframe,i))/2;
      //Print("medianPrice: "+medianPrice[i]);
      up[i]=medianPrice+(ATRMultiplier*atr);
      //Print("up: "+up[i]);
      dn[i]=medianPrice-(ATRMultiplier*atr);
      //Print("dn: "+dn[i]);
      trend[i]=1;

      if (iClose(Instrument,Timeframe,i)>up[i+1]) {
         trend[i]=1;
         if (trend[i+1] == -1) changeOfTrend = 1;
         //Print("trend: "+trend[i]);
         
      }
      else if (iClose(Instrument,Timeframe,i)<dn[i+1]) {
         trend[i]=-1;
         if (trend[i+1] == 1) changeOfTrend = 1;
         //Print("trend: "+trend[i]);
      }
      else if (trend[i+1]==1) {
         trend[i]=1;
         changeOfTrend = 0;       
      }
      else if (trend[i+1]==-1) {
         trend[i]=-1;
         changeOfTrend = 0;
      }

      if (trend[i]<0 && trend[i+1]>0) {
         flag=1;
         //Print("flag: "+flag);
      }
      else {
         flag=0;
         //Print("flagh: "+flag);
      }
      
      if (trend[i]>0 && trend[i+1]<0) {
         flagh=1;
         //Print("flagh: "+flagh);
      }
      else {
         flagh=0;
         //Print("flagh: "+flagh);
      }
      
      if (trend[i]>0 && dn[i]<dn[i+1])
         dn[i]=dn[i+1];
      
      if (trend[i]<0 && up[i]>up[i+1])
         up[i]=up[i+1];
      
      if (flag==1)
         up[i]=medianPrice+(ATRMultiplier*atr);
         
      if (flagh==1)
         dn[i]=medianPrice-(ATRMultiplier*atr);
         
      //-- Draw the indicator
      if(i==MaxBars) continue;
      if (trend[i]==1) {
         TrendUpTmp[i]=dn[i];
         if (changeOfTrend == 1) {
            TrendUpTmp[i+1] = TrendDownTmp[i+1];
            changeOfTrend = 0;
         }
      }
      else if (trend[i]==-1) {
         TrendDownTmp[i]=up[i];
         if (changeOfTrend == 1) {
            TrendDownTmp[i+1] = TrendUpTmp[i+1];
            changeOfTrend = 0;
         }
      }
   }
   WindowRedraw();
}


void TrailingStop(){
   for(int i=0;i<OrdersTotal();i++) {
      if(OrderSelect( i, SELECT_BY_POS, MODE_TRADES ) == false ) {
         int Error=GetLastError();
         string ErrorText=GetLastErrorText(Error);
         Print("ERROR - Unable to select the order - ",Error);
         Print("ERROR - ",ErrorText);
         break;
      }
      if(OnlyCurrentSymbol && OrderSymbol()!=Symbol()) continue;
      if(UseMagic && OrderMagicNumber()!=MagicNumber) continue;
      if(UseComment && StringFind(OrderComment(),CommentFilter)<0) continue;
      if(OnlyType!=All &&OrderType()!=OnlyType) continue;
      
      double NewSL=0;
      double NewTP=0;
      string Instrument=OrderSymbol();
      double ATRTrend=GetATRTrend(Instrument,StopATRTimeframe,SuperTrendShift);
      if(ATRTrend==0){
         Print("Not enough historical data, please load more candles for The selected timeframe");
         return;
      }
      //Print(OrderSymbol()," ",OrderMagicNumber()," ",ATRTrend);
      int eDigits = (int)MarketInfo(Instrument,MODE_DIGITS);
      double SLPrice=NormalizeDouble(OrderStopLoss(),eDigits);
      double TPPrice=NormalizeDouble(OrderTakeProfit(),eDigits);
      double Spread=MarketInfo(Instrument,MODE_SPREAD)*MarketInfo(Instrument,MODE_POINT);
      double StopLevel=MarketInfo(Instrument,MODE_STOPLEVEL)*MarketInfo(Instrument,MODE_POINT);
      //Print(StopLevel);
      //Print(Instrument," ",ATRTrend," ",StopLevel);
      if(OrderType()==OP_BUY && ATRTrend<MarketInfo(Instrument,MODE_BID)-StopLevel){
         NewSL=NormalizeDouble(ATRTrend,eDigits);
         NewTP=TPPrice;
         //Print(OrderSymbol()," ",OrderType()," Old SL=",SLPrice," New SL=",NewSL," Old TP=",TPPrice," New TP=",NewTP," Bid=",MarketInfo(OrderSymbol(),MODE_BID)," Ask=",MarketInfo(OrderSymbol(),MODE_ASK));
         if(NewSL>SLPrice+StopLevel || SLPrice==0){
            ModifyOrder(OrderTicket(),OrderOpenPrice(),NewSL,NewTP, Instrument);
         }
      }
      if(OrderType()==OP_SELL && ATRTrend>MarketInfo(Instrument,MODE_ASK)+StopLevel+Spread){
         NewSL=NormalizeDouble(ATRTrend+Spread,eDigits);
         NewTP=TPPrice;
         //Print(OrderSymbol()," ",OrderType()," Old SL=",SLPrice," New SL=",NewSL," Old TP=",TPPrice," New TP=",NewTP," Bid=",MarketInfo(OrderSymbol(),MODE_BID)," Ask=",MarketInfo(OrderSymbol(),MODE_ASK));
         if(NewSL<SLPrice-StopLevel || SLPrice==0){
            ModifyOrder(OrderTicket(),OrderOpenPrice(),NewSL,NewTP, Instrument);
         }
      }
   }
}


void ModifyOrder(int Ticket, double OpenPrice, double SLPrice, double TPPrice, string Instrument){
   if(OrderSelect(Ticket,SELECT_BY_TICKET)==false){
      int Error=GetLastError();
      string ErrorText=GetLastErrorText(Error);
      Print("ERROR - SELECT TICKET - error selecting order ",Ticket," return error: ",Error);
      return;
   }
   int eDigits = (int)MarketInfo(Instrument, MODE_DIGITS);
   SLPrice=NormalizeDouble(SLPrice,eDigits);
   TPPrice=NormalizeDouble(TPPrice,eDigits);
   for(int i=1; i<=OrderOpRetry; i++){
      bool res=OrderModify(Ticket,OpenPrice,SLPrice,TPPrice,0,Blue);
      if(res){
         Print("TRADE - UPDATE SUCCESS - Order ",Ticket," new stop loss ",SLPrice," new take profit ",TPPrice);
         NotifyStopLossUpdate(Ticket,SLPrice, Instrument);
         break;
      }
      else{
         int Error=GetLastError();
         string ErrorText=GetLastErrorText(Error);
         Print("ERROR - UPDATE FAILED - error modifying order ",Ticket," return error: ",Error," Open=",OpenPrice,
               " Old SL=",OrderStopLoss()," Old TP=",OrderTakeProfit(),
               " New SL=",SLPrice," New TP=",TPPrice," Bid=",MarketInfo(Instrument,MODE_BID)," Ask=",MarketInfo(Instrument,MODE_ASK));
         Print("ERROR - ",ErrorText);
      } 
   }
   return;
}


void NotifyStopLossUpdate(int OrderNumber, double SLPrice, string Instrument){
   if(!EnableNotify) return;
   if(!SendAlert && !SendApp && !SendEmail) return;
   int eDigits = (int)MarketInfo(Instrument, MODE_DIGITS);
   string EmailSubject=IndicatorName+" "+Instrument+" Notification ";
   string EmailBody="\r\n"+AccountCompany()+" - "+AccountName()+" - "+IntegerToString(AccountNumber())+"\r\n\r\n"+IndicatorName+" Notification for " + Instrument + "\r\n\r\n";
   EmailBody+="The Stop Loss for order " + IntegerToString(OrderNumber) + " was moved to "+DoubleToString(SLPrice,eDigits)+"\r\n\r\n";
   string AlertText=IndicatorName+" - " + Instrument + " Notification\r\n";
   AlertText+="The Stop Loss for order " + IntegerToString(OrderNumber) + " was moved to "+DoubleToString(SLPrice,eDigits)+"";
   string AppText=AccountCompany()+" - "+AccountName()+" - "+IntegerToString(AccountNumber())+" - "+IndicatorName+" - " + Instrument + " - ";
   AppText+="The Stop Loss for order " + IntegerToString(OrderNumber) + " was moved to "+DoubleToString(SLPrice,eDigits)+"";
   if(SendAlert) Alert(AlertText);
   if(SendEmail){
      if(!SendMail(EmailSubject,EmailBody)) Print("Error sending email "+IntegerToString(GetLastError()));
   }
   if(SendApp){
      if(!SendNotification(AppText)) Print("Error sending notification "+IntegerToString(GetLastError()));
   }
   datetime LastNotification=TimeCurrent();
   Print(IndicatorName+"-" + Instrument + " last notification sent "+TimeToString(LastNotification));
}


string PanelBase=IndicatorName+"-P-BAS";
string PanelLabel=IndicatorName+"-P-LAB";
string PanelEnableDisable=IndicatorName+"-P-ENADIS";

int PanelMovX=50;
int PanelMovY=20;
int PanelLabX=150;
int PanelLabY=PanelMovY;
int PanelRecX=PanelLabX+4;

void DrawPanel(){
   CleanPanel();
   int Rows=1;
   ObjectCreate(0,PanelBase,OBJ_RECTANGLE_LABEL,0,0,0);
   ObjectSet(PanelBase,OBJPROP_XDISTANCE,Xoff);
   ObjectSet(PanelBase,OBJPROP_YDISTANCE,Yoff);
   ObjectSetInteger(0,PanelBase,OBJPROP_XSIZE,PanelRecX);
   ObjectSetInteger(0,PanelBase,OBJPROP_YSIZE,(PanelMovY+2)*1+2);
   ObjectSetInteger(0,PanelBase,OBJPROP_BGCOLOR,White);
   ObjectSetInteger(0,PanelBase,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelBase,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelBase,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelBase,OBJPROP_FONTSIZE,8);
   ObjectSet(PanelBase,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelBase,OBJPROP_COLOR,clrBlack);
      
   ObjectCreate(0,PanelLabel,OBJ_EDIT,0,0,0);
   ObjectSet(PanelLabel,OBJPROP_XDISTANCE,Xoff+2);
   ObjectSet(PanelLabel,OBJPROP_YDISTANCE,Yoff+2);
   ObjectSetInteger(0,PanelLabel,OBJPROP_XSIZE,PanelLabX);
   ObjectSetInteger(0,PanelLabel,OBJPROP_YSIZE,PanelLabY);
   ObjectSetInteger(0,PanelLabel,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelLabel,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelLabel,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelLabel,OBJPROP_READONLY,true);
   ObjectSetInteger(0,PanelLabel,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,PanelLabel,OBJPROP_TOOLTIP,"Supertrend Trailing Stop Loss By MQLTA");
   ObjectSetString(0,PanelLabel,OBJPROP_TEXT,"MQLTA STTS");
   ObjectSetString(0,PanelLabel,OBJPROP_FONT,"Consolas");
   ObjectSetInteger(0,PanelLabel,OBJPROP_FONTSIZE,10);
   ObjectSet(PanelLabel,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelLabel,OBJPROP_COLOR,clrNavy);
   ObjectSetInteger(0,PanelLabel,OBJPROP_BGCOLOR,clrKhaki);
   ObjectSetInteger(0,PanelLabel,OBJPROP_BORDER_COLOR,clrBlack);
   
   string EnableDisabledText="";
   color EnableDisabledColor=clrNavy;
   color EnableDisabledBack=clrKhaki;
   if(EnableTrailing){
      EnableDisabledText="TRAILING ENABLED";
      EnableDisabledColor=clrWhite;
      EnableDisabledBack=clrDarkGreen;
   }
   else{
      EnableDisabledText="TRAILING DISABLED";
      EnableDisabledColor=clrWhite;
      EnableDisabledBack=clrDarkRed;
   }
   
   ObjectCreate(0,PanelEnableDisable,OBJ_EDIT,0,0,0);
   ObjectSet(PanelEnableDisable,OBJPROP_XDISTANCE,Xoff+2);
   ObjectSet(PanelEnableDisable,OBJPROP_YDISTANCE,Yoff+(PanelMovY+1)*Rows+2);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_XSIZE,PanelLabX);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_YSIZE,PanelLabY);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_READONLY,true);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_FONTSIZE,8);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,PanelEnableDisable,OBJPROP_FONT,"Consolas");
   ObjectSetString(0,PanelEnableDisable,OBJPROP_TOOLTIP,"Click to Enable or Disable the Trailing Stop Feature");
   ObjectSetString(0,PanelEnableDisable,OBJPROP_TEXT,EnableDisabledText);
   ObjectSet(PanelEnableDisable,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_COLOR,EnableDisabledColor);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_BGCOLOR,EnableDisabledBack);
   ObjectSetInteger(0,PanelEnableDisable,OBJPROP_BORDER_COLOR,clrBlack);
   Rows++;

   
   ObjectSetInteger(0,PanelBase,OBJPROP_XSIZE,PanelRecX);
   ObjectSetInteger(0,PanelBase,OBJPROP_YSIZE,(PanelMovY+1)*Rows+3);
}


void CleanPanel(){
   int Window=0;
   for(int i=ObjectsTotal(ChartID(),Window,-1)-1;i>=0;i--){
      if(StringFind(ObjectName(i),IndicatorName+"-P-",0)>=0){
         ObjectDelete(ObjectName(i));
      }
   }
}


void ChangeTrailingEnabled(){
   if(EnableTrailing==false){
      if(IsTradeAllowed()) EnableTrailing=true;
      else{
         MessageBox("You need to first enable Live Trading in your Metatrader options","WARNING",MB_OK);
      }
   }
   else EnableTrailing=false;
   DrawPanel();
}